import { container_paths, containers_path, state_config_path } from "./paths.ts";
import { exists, log_from } from "./utils.ts";
import { z } from "https://deno.land/x/zod@v3.22.4/mod.ts";

const log = log_from("config");

export function new_container_config(name: string): ContainerConfig {
	return {
		name,
		version: 0,
		redirects: [] as [number, number][],
	};
}

export type ContainerConfig = ReturnType<typeof parse_container_config>;
export function parse_container_config(json: string) {
	return z.object({
		name: z.string(),
		version: z.number(),
		redirects: z.array(
			z
				.tuple([
					z.number(),
					z.number(),
				])
				.or(z.tuple([
					z.number(),
					z.number(),
					z.literal("tcp").or(z.literal("udp")),
				])),
		),
	}).parse(JSON.parse(json));
}

export async function load_container_config(name: string) {
	const config_path = container_paths(name).configuration;
	log("loading config for", name);
	if (!exists(config_path)) return null;
	const content = await Deno.readTextFile(config_path);
	const read = parse_container_config(content);
	const default_ = new_container_config(name);
	if (read.version < default_.version) throw new Error("read conf version is outdated");
	return { ...default_, ...read } as ContainerConfig;
}

export async function save_container_config(config: ContainerConfig) {
	log("saving config of", config.name);
	const config_path = container_paths(config.name).configuration;
	const serialized = JSON.stringify(config, null, 4);
	await Deno.writeTextFile(config_path, serialized);
}

export async function load_all_container_configs() {
	const names = Array.from(Deno.readDirSync(containers_path()))
		.filter((e) => e.isDirectory)
		.map((e) => e.name);
	const results = await Promise.all(names.map((name) => load_container_config(name)));
	return results.filter((item) => item != null) as ContainerConfig[];
}
