// import { blob } from "https://deno.land/std@0.208.0/streams/mod.ts"

import { ContainerConfig } from "./config.ts";
import { base_paths } from "./paths.ts";
import { lines, log_from } from "./utils.ts";

export type Base = (context: BaseContext) => unknown;
const log = log_from("BUILDING");

export type BaseContext = ReturnType<typeof new_container_context>;
export function new_container_context(root_dir: string, config: ContainerConfig, known_containers: ContainerConfig[]) {
	return {
		root_dir,
		config,
		sh: async (script: string) => {
			log(`running '${script.replace("\n", " ").slice(0, 20)}...'`);
			const process = new Deno.Command("sudo", {
				args: ["sh", "-c", "set -e;" + script],
				stdin: "null",
				stdout: "piped",
			}).spawn();
			log_lines_prefixed("  | ", process.stdout);
			const res = await process.status;
			if (!res.success) throw new Error("process failed");
		},
		sh_in: async (script: string) => {
			log(`inside '${root_dir}' running '${script.replace("\n", " ").slice(0, 20)}...'`);
			const process = new Deno.Command("sudo", {
				args: ["systemd-nspawn", `--directory=${root_dir}`, "-P"],
				stdin: "piped",
				stdout: "piped",
			}).spawn();
			write_all(process.stdin, "set -e;" + script + "\nexit 0\n");
			log_lines_prefixed("  | ", process.stdout);
			const res = await process.status;
			if (!res.success) throw new Error("process failed");
		},
		redirect: (from: number, to: number) => {
			log("redirects", from, "to", to);
			config.redirects.push([from, to]);
		},
		available_port: (target: number) => {
			const used_by_known_containers = known_containers.map((c) => c.redirects[0]).flat();
			const used_by_self = config.redirects.map((r) => r[0]);
			const used = [...used_by_known_containers, ...used_by_self];
			let result = target;
			while (used.includes(result)) result += 1;
			return result;
		},
	};
}

async function log_lines_prefixed(prefix: string, stream: ReadableStream<Uint8Array>) {
	for await (const line of lines(stream)) {
		console.log(prefix + line);
	}
}

async function write_all(stream: WritableStream<Uint8Array>, text: string) {
	await stream.getWriter().write(new TextEncoder().encode(text));
}

export async function load_base(name: string) {
	const paths = base_paths(name);
	const module = await import(paths.module);
	const build = module["build"] as (context: BaseContext) => unknown;
	if (build === undefined) throw new Error("module does not contain a build procedure");
	return { build };
}
